from sqlalchemy import *
from sqlalchemy import orm
from sqlalchemy.orm import create_session, relation

from sqlalchemy.ext.sessioncontext import SessionContext
from sqlalchemy.ext.assignmapper import assign_mapper

from invirt.database import record

__all__ = ['meta',
           'session',
           'clear_cache',
           'machine_table',
           'machine_access_table',
           'nic_table',
           'disk_table',
           'types_table',
           'cdroms_table',
           'mirrors_table',
           'autoinstalls_table',
           'owners_table',
           'Machine',
           'MachineAccess',
           'NIC',
           'Disk',
           'Type',
           'CDROM',
           'Mirror',
           'Autoinstall',
           'Owner',
           'or_',
           ]

meta = ThreadLocalMetaData()
session = orm.scoped_session(orm.sessionmaker(transactional=False, autoflush=False))

machine_table = Table('machines', meta,
       Column('machine_id', Integer, primary_key=True, nullable=False),
       Column('name', String, nullable=False),
       Column('description', String, nullable=False),
       Column('memory', Integer, nullable=False),
       Column('owner', String, nullable=False),
       Column('contact', String, nullable=False),
       Column('uuid', String, nullable=False),
       Column('administrator', String, nullable=True, default=None),
       Column('type_id', String, ForeignKey('types.type_id'), nullable=False),
       Column('autorestart', Boolean, nullable=False, default=False),
       Column('cpus', Integer, nullable=False, default=1),
       Column('adminable', Boolean, nullable=False, default=False))

nic_table = Table('nics', meta,
       Column('machine_id', Integer, ForeignKey('machines.machine_id'), nullable=True),
       Column('mac_addr', String, nullable=False, primary_key=True),
       Column('ip', String, nullable=False, unique=True),
       Column('hostname', String, nullable=True),
       Column('reusable', Boolean, nullable=False, default=True))

disk_table = Table('disks', meta,
       Column('machine_id', Integer, ForeignKey('machines.machine_id'), nullable=False),
       Column('guest_device_name', String, nullable=False),
       Column('size', Integer, nullable=False),
       PrimaryKeyConstraint('machine_id', 'guest_device_name'))

types_table = Table('types', meta,
       Column('type_id', String, primary_key=True, nullable=False),
       Column('description', String, nullable=False),
       Column('hvm', Boolean, nullable=False),
       Column('apic', Boolean, nullable=False),
       Column('acpi', Boolean, nullable=False),
       Column('pae', Boolean, nullable=False))

mirrors_table = Table('mirrors', meta,
       Column('mirror_id', String, primary_key=True, nullable=False),
       Column('uri_prefix', String, nullable=False))

cdroms_table = Table('cdroms', meta,
       Column('cdrom_id', String, primary_key=True, nullable=False),
       Column('description', String, nullable=False),
       Column('mirror_id', String, ForeignKey('mirrors.mirror_id')),
       Column('uri_suffix', String))

autoinstalls_table = Table('autoinstalls', meta,
       Column('autoinstall_id', String, primary_key=True, nullable=False),
       Column('description', String, nullable=False),
       Column('type_id', String, ForeignKey('types.type_id'), nullable=False),
       Column('distribution', String, nullable=False),
       Column('mirror', String, nullable=False),
       Column('arch', String, nullable=False))

owners_table = Table('owners', meta,
       Column('owner_id', String, primary_key=True, nullable=False),
       Column('ram_quota_total', Integer, nullable=True, default=None),
       Column('ram_quota_single', Integer, nullable=True, default=None),
       Column('disk_quota_total', Integer, nullable=True, default=None),
       Column('disk_quota_single', Integer, nullable=True, default=None),
       Column('vms_quota_total', Integer, nullable=True, default=None),
       Column('vms_quota_active', Integer, nullable=True, default=None))

machine_access_table = Table('machine_access', meta,
       Column('machine_id', Integer, ForeignKey('machines.machine_id', ondelete='CASCADE'), nullable=False, index=True),
       Column('user', String, nullable=False, index=True),
       PrimaryKeyConstraint('machine_id', 'user'))

admins_table = Table('admins', meta,
       Column('user', String, nullable=False, index=True, primary_key=True))

class Machine(record.Record):
    _identity_field = 'name'

class MachineAccess(record.Record):
    pass

class NIC(record.Record):
    pass

class Disk(record.Record):
    pass

class Type(record.Record):
    _identity_field = 'type_id'

class Mirror(record.Record):
    _identity_field = 'mirror_id'

class CDROM(record.Record):
    _identity_field = 'cdrom_id'

class Autoinstall(record.Record):
    _identity_field = 'autoinstall_id'

class Admin(record.Record):
    _identity_field = 'user'

from invirt.database.owner import Owner

session.mapper(Machine, machine_table,
              properties={'nics': relation(NIC, backref="machine"),
                          'disks': relation(Disk, backref="machine"),
                          'type': relation(Type),
                          'acl': relation(MachineAccess, backref="machine", passive_deletes=True, cascade="all, delete-orphan")});
session.mapper(MachineAccess, machine_access_table)
session.mapper(NIC, nic_table)
session.mapper(Disk, disk_table)
session.mapper(Type, types_table)
session.mapper(Mirror, mirrors_table)
session.mapper(CDROM, cdroms_table,
               properties={'mirror': relation(Mirror, backref="cdroms")})
session.mapper(Autoinstall, autoinstalls_table)
session.mapper(Owner, owners_table)
session.mapper(Admin, admins_table)

def clear_cache():
    """Clear sqlalchemy's cache
    """

    session.clear()
